/*
 * $Header: /home/cvspublic/jakarta-commons-sandbox/resources/src/java/org/apache/commons/resources/Resources.java,v 1.9 2003/02/28 02:50:34 dgraham Exp $
 * $Revision: 1.9 $
 * $Date: 2003/02/28 02:50:34 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.commons.resources;


import java.io.InputStream;
import java.io.Reader;
import java.io.Serializable;
import java.util.Locale;
import java.util.TimeZone;


/**
 * <p>An abstract representation of a set of internationalized resources,
 * which are arbitrary objects identified by a unique String <code>key</code>.
 * Localized versions of the resources (based on Locale and optional
 * TimeZone parameters) can be retrieved in a variety of formats.</p>
 *
 * <p>When presented with an invalid <code>key</code> value, resource
 * getter methods will behave differently based on the current value of
 * the <code>returnNull</code> property for this {@link Resources} instance.
 * If the property value is <code>true</code>, the getter method will return
 * <code>null</code> (which may be ambiguous if <code>null</code> is a valid
 * resource value).  If the property value is <code>false</code>, a
 * <code>ResourcesKeyException</code> will be thrown instead.</p>
 *
 * <p>Different implementations of the {@link Resources} interface support
 * a variety of mechanisms for acquiring the data content represented by
 * the keys.  Examples could include property files, XML files, databases, web
 * application resources, and other specialized approaches as desired.</p>
 *
 * <p>Different implementations of the {@link Resources} interface may apply
 * different semantics to the use of the <code>locale</code> and/or
 * <code>timeZone</code> attributes used to perform localization.  Consult
 * the documentation for the specific {@link Resources} implementation you
 * are using for the specific details of your implementation.</p>
 *
 * <p>Developers implementing {@link Resources} should extend the
 * {@link org.apache.commons.resources.impl.ResourcesBase} class,
 * and override the necessary methods,
 * to shield themselves from future changes that may occur in this interface.
 *
 * @author Mike Schachter (mschachter@hp.com)
 * @author Craig R. McClanahan
 * @version $Revision: 1.9 $ $Date: 2003/02/28 02:50:34 $
 */

public interface Resources extends Serializable {


    // ------------------------------------------------------ Lifecycle Methods


    /**
     * <p>This must be called to initialize the data content of this
     * {@link Resources} instance, before any of the <code>getXxx()</code>
     * methods are called.</p>
     *
     * @exception ResourcesException if an error occurs during initialization
     */
    public void init() throws ResourcesException;


    /**
     * <p>This method must be called when the manager of this resource
     * decides that it's no longer needed.  After this method is called,
     * no further calls to any of the <code>getXxx()</code> methods are
     * allowed.</p>
     *
     * @exception ResourcesException if an error occurs during finalization
     */
    public void destroy() throws ResourcesException;


    // ------------------------------------------------------------- Properties


    /**
     * <p>Return the logical name of this {@link Resources} instance.</p>
     */
    public String getName();


    /**
     * <p>Return <code>true</code> if resource getter methods will return
     * <code>null</code> instead of throwing an exception on invalid
     * key values.</p>
     */
    public boolean isReturnNull();


    /**
     * <p>Set a flag determining whether resource getter methods should
     * return <code>null</code> instead of throwing an exception on
     * invalid key values.</p>
     *
     * @param returnNull The new flag value
     */
    public void setReturnNull(boolean returnNull);


    // ---------------------------------------------- Content Retrieval Methods


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * byte array, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public byte[] getBytes(String key, Locale locale, TimeZone timeZone);


    /**
     * <p>Return the content for the specified <code>key</code> as an
     * InputStream, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public InputStream getInputStream(String key, Locale locale,
                                      TimeZone timeZone);


    /**
     * <p>Return the content for the specified <code>key</code> as an
     * Object, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public Object getObject(String key, Locale locale, TimeZone timeZone);


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * Reader, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public Reader getReader(String key, Locale locale, TimeZone timeZone);


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * String, localized based on the specified <code>locale</code>
     * and/or <code>timeZone</code>.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @param timeZone TimeZone with which to localize retrieval,
     *  or <code>null</code> for the default TimeZone
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public String getString(String key, Locale locale, TimeZone timeZone);


}


